//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Catalog/ItemCatalog.cpp
//! @brief     Implements class ItemCatalog
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Catalog/ItemCatalog.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Ax/PointwiseAxisItem.h"
#include "GUI/Model/Data/IntensityDataItem.h"
#include "GUI/Model/Data/RealDataItem.h"
#include "GUI/Model/Data/SpecularDataItem.h"
#include "GUI/Model/Fit/FitParameterContainerItem.h"
#include "GUI/Model/Fit/FitParameterItem.h"
#include "GUI/Model/Fit/FitParameterLinkItem.h"
#include "GUI/Model/Fit/FitSuiteItem.h"
#include "GUI/Model/Fit/JobItem.h"
#include "GUI/Model/Fit/MinimizerItem.h"
#include "GUI/Model/Fit/ParameterTreeItems.h"
#include "GUI/Model/Instrument/BackgroundItems.h"
#include "GUI/Model/Instrument/BeamAngleItems.h"
#include "GUI/Model/Instrument/BeamWavelengthItem.h"
#include "GUI/Model/Instrument/FootprintItems.h"
#include "GUI/Model/Instrument/InstrumentItems.h"
#include "GUI/Model/Instrument/RectangularDetectorItem.h"
#include "GUI/Model/Instrument/ResolutionFunctionItems.h"
#include "GUI/Model/Instrument/SpecularBeamInclinationItem.h"
#include "GUI/Model/Instrument/SphericalDetectorItem.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/Model/Mask/ProjectionItems.h"
#include "GUI/Model/Plot/Data1DViewItem.h"
#include "GUI/Model/Plot/DataProperties.h"
#include "GUI/Model/Plot/DataPropertyContainer.h"
#include "GUI/Model/Sample/FormFactorItems.h"
#include "GUI/Model/Sample/InterferenceItems.h"
#include "GUI/Model/Sample/MesoCrystalItem.h"
#include "GUI/Model/Sample/ParticleCompositionItem.h"
#include "GUI/Model/Sample/ParticleCoreShellItem.h"
#include "GUI/Model/Sample/ParticleItem.h"
#include "GUI/Model/Sample/ParticleLayoutItem.h"
#include "GUI/Model/Session/PropertyItem.h"
#include "GUI/Model/Session/RealLimitsItems.h"

ItemCatalog::ItemCatalog()
{
    addItem<GISASInstrumentItem>();
    addItem<OffSpecularInstrumentItem>();
    addItem<SpecularInstrumentItem>();
    addItem<GISASBeamItem>();
    addItem<SpecularBeamItem>();
    addItem<NoBackgroundItem>();
    addItem<ConstantBackgroundItem>();
    addItem<PoissonNoiseBackgroundItem>();
    addItem<FootprintNoneItem>();
    addItem<FootprintGaussianItem>();
    addItem<FootprintSquareItem>();
    addItem<VectorItem>();
    addItem<PropertyItem>();


    addItem<SphericalDetectorItem>();
    addItem<RectangularDetectorItem>();

    addItem<DistributionNoneItem>();
    addItem<DistributionGateItem>();
    addItem<DistributionLorentzItem>();
    addItem<DistributionGaussianItem>();
    addItem<DistributionLogNormalItem>();
    addItem<DistributionCosineItem>();
    addItem<DistributionTrapezoidItem>();

    addItem<JobItem>();

    addItem<IntensityDataItem>();
    addItem<SpecularDataItem>();
    addItem<Data1DViewItem>();
    addItem<DataPropertyContainer>();
    addItem<Data1DProperties>();

    addItem<BasicAxisItem>();
    addItem<PointwiseAxisItem>();
    addItem<AmplitudeAxisItem>();

    addItem<BeamWavelengthItem>();
    addItem<BeamAzimuthalAngleItem>();
    addItem<BeamInclinationAngleItem>();
    addItem<SpecularBeamInclinationItem>();
    addItem<SpecularBeamWavelengthItem>();

    addItem<ResolutionFunctionNoneItem>();
    addItem<ResolutionFunction2DGaussianItem>();

    addItem<MaskContainerItem>();
    addItem<RectangleItem>();
    addItem<PolygonPointItem>();
    addItem<PolygonItem>();
    addItem<VerticalLineItem>();
    addItem<HorizontalLineItem>();
    addItem<EllipseItem>();
    addItem<MaskAllItem>();

    addItem<RegionOfInterestItem>();

    addItem<GroupItem>();

    addItem<FitParameterContainerItem>();
    addItem<FitParameterItem>();
    addItem<FitParameterLinkItem>();
    addItem<FitSuiteItem>();

    addItem<RealDataItem>();

    addItem<MinimizerContainerItem>();
    addItem<MinuitMinimizerItem>();
    addItem<GSLMultiMinimizerItem>();
    addItem<GeneticMinimizerItem>();
    addItem<SimAnMinimizerItem>();
    addItem<GSLLMAMinimizerItem>();

    addItem<LimitlessItem>();
    addItem<PositiveItem>();
    addItem<NonnegativeItem>();
    addItem<LowerLimitedItem>();
    addItem<UpperLimitedItem>();
    addItem<LimitedItem>();

    addItem<ProjectionContainerItem>();

    addItem<DepthProbeInstrumentItem>();
}

ItemCatalog& ItemCatalog::instance()
{
    static ItemCatalog catalog;
    return catalog;
}

std::unique_ptr<SessionItem> ItemCatalog::createItemPtr(const QString& modelType) const
{
    ASSERT(m_data.contains(modelType));
    return m_data.createItemPtr(modelType);
}

template <typename T> void ItemCatalog::addItem()
{
    m_data.registerItem(T::M_TYPE, [] { return new T(); });
}
