//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/ParticleLayoutItem.h
//! @brief     Defines class ParticleLayoutItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_PARTICLELAYOUTITEM_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_PARTICLELAYOUTITEM_H

#include "GUI/Model/Group/SelectionProperty.h"
#include "GUI/Model/Sample/InterferenceItems.h"
#include "GUI/Model/Types/DoubleProperty.h"
#include <QUuid>
#include <memory>


class ItemWithParticles;
class QXmlStreamReader;
class QXmlStreamWriter;
class Serializer;

class ParticleLayoutItem {
public:
    ParticleLayoutItem();

    //! The density value which belonging only to the layout.
    //!
    //! This is the editable value. If an interference is present, this value may not be the one to
    //! be used for building the domain layout or to be presented. For the correct value-to-use,
    //! whether an interference is present or not, use totalDensity().
    DoubleDescriptor ownDensity() const;

    //! The real density.
    //!
    //! Returns ownDensity() if the interference is not influencing the density. If the
    //! interference is defining the density, this is the interference-calculated density.
    DoubleDescriptor totalDensity() const;
    double totalDensityValue() const;
    DoubleDescriptor weight() const;

    //! The particles this layout contains.
    //!
    //! Note that this is not a hierarchical search! Only the topmost particles are returned, not
    //! their contained particles. For a hierarchical search use containedItemsWithParticles
    QVector<ItemWithParticles*> particles() const;
    void addParticle(ItemWithParticles* particle);
    void removeParticle(ItemWithParticles* particle);

    //! Return full hierarchical contained items with particles.
    //!
    //! \sa particles()
    QVector<ItemWithParticles*> containedItemsWithParticles() const;

    SelectionDescriptor<InterferenceItem*> interference() const;
    void setInterference(InterferenceItem* interference);
    void removeInterference();

    //! Returns whether total density is defined by the currently selected interference.
    //!
    //! Two dimensional interference calculates density automatically; in these cases the "own"
    //! total density value should not be edited but set by the one from the interference.
    bool totalDensityIsDefinedByInterference() const;

    void serialize(Serializer& s);

private:
    DoubleProperty m_ownDensity;
    DoubleProperty m_weight;
    SelectionProperty<InterferenceItem*> m_interference;
    QVector<ItemWithParticles*> m_particles;
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_PARTICLELAYOUTITEM_H
