//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/XML/Serializer.cpp
//! @brief     Implements class Serializer
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/XML/Serializer.h"
#include "GUI/Model/Types/DoubleProperty.h"
#include "GUI/Model/Types/UIntProperty.h"
#include "GUI/Model/Types/VectorProperty.h"
#include "GUI/Model/XML/SessionXML.h"
#include "GUI/Support/Data/XML.h"
#include "GUI/Util/DeserializationException.h"
#include <QColor>

namespace {

namespace Tags {
constexpr auto Id("id");
constexpr auto Value("value");
constexpr auto Decimals("decimals");
constexpr auto Version("version");
} // namespace Tags
} // namespace

void Serializer::assertVersion(unsigned expectedVersion) const
{
    if (!m_r)
        return;

    const auto foundAsStr = m_r->attributes().value(Tags::Version);
    const unsigned found = foundAsStr.isEmpty() ? 0 : foundAsStr.toUInt();
    if (found < expectedVersion)
        throw DeserializationException::tooOld();
    if (found > expectedVersion)
        throw DeserializationException::tooNew();
}

bool Serializer::isReading() const
{
    return m_r;
}

void Serializer::rw(const QString& tag, QString& val)
{
    if (m_w) {
        m_w->writeStartElement(tag);
        m_w->writeAttribute(Tags::Value, val);
        m_w->writeEndElement();
    } else {
        QString tagBeforeReadNextStartElement = m_r->name().toString();
        m_r->readNextStartElement();
        assertCurrentTag(tag);
        GUI::Session::XML::readAttribute(m_r, Tags::Value, &val);
        gotoEndElementOfTag(tag);
    }
}

void Serializer::rw(DoubleProperty& d)
{
    if (m_w) {
        m_w->writeStartElement(d.persistentTag());
        GUI::Session::XML::writeAttribute(m_w, Tags::Value, d.get());
        m_w->writeAttribute(Tags::Id, d.uid());
        GUI::Session::XML::writeAttribute(m_w, Tags::Decimals, d.decimals());
        m_w->writeEndElement();
    } else {
        QString tagBeforeReadNextStartElement = m_r->name().toString();
        m_r->readNextStartElement();
        assertCurrentTag(d.persistentTag());
        const double val = m_r->attributes().value(Tags::Value).toDouble();
        const QString uid = m_r->attributes().value(Tags::Id).toString();
        unsigned decimals = m_r->attributes().value(Tags::Decimals).toUInt();
        d.set(val);
        d.setUid(uid);
        d.setDecimals(decimals);
        gotoEndElementOfTag(d.persistentTag());
    }
}

void Serializer::rw(UIntProperty& d)
{
    if (m_w) {
        m_w->writeStartElement(d.persistentTag());
        GUI::Session::XML::writeAttribute(m_w, Tags::Value, d.get());
        GUI::Session::XML::writeUid(m_w, d.uid());
        m_w->writeEndElement();
    } else {
        QString tagBeforeReadNextStartElement = m_r->name().toString();
        m_r->readNextStartElement();
        assertCurrentTag(d.persistentTag());
        const uint val = m_r->attributes().value(Tags::Value).toUInt();
        const QString uid = m_r->attributes().value(Tags::Id).toString();
        d.set(val);
        d.setUid(uid);
        gotoEndElementOfTag(d.persistentTag());
    }
}

void Serializer::rw(VectorProperty& d)
{
    if (m_w) {
        m_w->writeStartElement(d.persistentTag());
        GUI::Session::XML::writeUid(m_w, d.uid());
        GUI::Session::XML::writeAttribute(m_w, Tags::Value, d.get());
        m_w->writeEndElement();
    } else {
        QString tagBeforeReadNextStartElement = m_r->name().toString();
        m_r->readNextStartElement();
        assertCurrentTag(d.persistentTag());
        R3 vec;
        QString uid;
        GUI::Session::XML::readAttribute(m_r, Tags::Value, &vec);
        GUI::Session::XML::readAttribute(m_r, Tags::Id, &uid);
        d.set(vec);
        d.setUid(uid);
        gotoEndElementOfTag(d.persistentTag());
    }
}

void Serializer::rw(const QString& tag, QColor& col)
{
    if (m_w) {
        m_w->writeStartElement(tag);
        m_w->writeAttribute(Tags::Value, col.isValid() ? col.name(QColor::HexArgb) : "");
        m_w->writeEndElement();
    } else {
        QString tagBeforeReadNextStartElement = m_r->name().toString();
        m_r->readNextStartElement();
        assertCurrentTag(tag);
        QString colName = m_r->attributes().value(Tags::Value).toString();
        col = QColor(colName);
        gotoEndElementOfTag(tag);
    }
}

void Serializer::rw(const QString& tag, bool& val)
{
    if (m_w) {
        m_w->writeStartElement(tag);
        m_w->writeAttribute(Tags::Value, val ? "1" : "0");
        m_w->writeEndElement();
    } else {
        QString tagBeforeReadNextStartElement = m_r->name().toString();
        m_r->readNextStartElement();
        assertCurrentTag(tag);
        val = m_r->attributes().value(Tags::Value).toUInt() > 0;
        gotoEndElementOfTag(tag);
    }
}

void Serializer::assertCurrentTag(const QString& expectedTag) const
{
    ASSERT(m_r);

    if (m_r->name() != expectedTag) {
#ifdef _DEBUG
        // to simplify debugging: what is the current tag
        QString foundTag = m_r->name().toString();
        Q_UNUSED(foundTag);
#endif
        throw DeserializationException::streamError();
    }
}

void Serializer::assertCurrentToken(QXmlStreamReader::TokenType token) const
{
    if (m_r->tokenType() != token)
        throw DeserializationException::streamError();
}

void Serializer::gotoEndElementOfTag(const QString& tag)
{
    if (m_r->name() != tag) {
        if (!m_r->isEndElement())
            m_r->skipCurrentElement();
        m_r->skipCurrentElement();
    }
    assertCurrentTag(tag);
    if (!m_r->isEndElement())
        m_r->skipCurrentElement();

    assertCurrentToken(QXmlStreamReader::EndElement);
    assertCurrentTag(tag);
}

void Serializer::goToStartElementOfTag(const QString& tag)
{
    ASSERT(m_r);
    while (m_r->name() != tag && !m_r->hasError())
        m_r->readNextStartElement();
    assertCurrentToken(QXmlStreamReader::StartElement);
}
