//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Particle/IBornFF.cpp
//! @brief     Implements interface class IBornFF.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Particle/IBornFF.h"
#include "Base/Util/Algorithms.h"
#include "Base/Util/PyFmt.h"
#include "Base/Util/StringUtils.h"
#include "Base/Vector/RotMatrix.h"
#include "Base/Vector/WavevectorInfo.h"
#include "Sample/Scattering/Rotations.h"
#include "Sample/Shapes/IShape3D.h"
#include <stdexcept>
#include <utility>

IBornFF::IBornFF() = default;

IBornFF::IBornFF(const std::vector<double>& PValues) : ISampleNode(PValues) {}

IBornFF::~IBornFF() = default;

std::string IBornFF::shapeName() const
{
    if (className().substr(0, 10) == "FormFactor")
        return className().substr(10);
    return className();
}

complex_t IBornFF::theFF(const WavevectorInfo& wavevectors) const
{
    return formfactor_at_bottom(wavevectors.getQ());
}

Eigen::Matrix2cd IBornFF::thePolFF(const WavevectorInfo& wavevectors) const
{
    return formfactor_pol(wavevectors.getQ());
}

double IBornFF::bottomZ(const IRotation* rotation) const
{
    if (!m_shape3D)
        throw std::runtime_error("Bug: Form factor has no m_shape3D, cannot compute bottom z");
    return BottomZ(m_shape3D->vertices(), rotation);
}

double IBornFF::topZ(const IRotation* rotation) const
{
    if (!m_shape3D)
        throw std::runtime_error("Bug: Form factor has no m_shape3D, cannot compute top z");
    return TopZ(m_shape3D->vertices(), rotation);
}

bool IBornFF::canSliceAnalytically(const IRotation* rotation) const
{
    return !rotation || rotation->zInvariant();
}

std::string IBornFF::pythonConstructor() const
{
    std::vector<std::pair<double, std::string>> arguments;
    for (size_t i = 0; i < parDefs().size(); i++)
        arguments.emplace_back(m_P[i], parDefs()[i].unit);

    return Py::Fmt::printFunction(className(), arguments);
}

Eigen::Matrix2cd IBornFF::formfactor_pol(C3 q) const
{
    return formfactor_at_bottom(q) * Eigen::Matrix2cd::Identity();
}

double IBornFF::volume() const
{
    return std::abs(formfactor_at_bottom(C3()));
}

double IBornFF::BottomZ(const std::vector<R3>& vertices, const IRotation* rotation)
{
    ASSERT(vertices.size());
    return BaseUtils::algo::min_value(
        vertices.begin(), vertices.end(), [&](const R3& vertex) -> double {
            return (rotation ? rotation->transformed(vertex) : vertex).z();
        });
}

double IBornFF::TopZ(const std::vector<R3>& vertices, const IRotation* rotation)
{
    ASSERT(vertices.size());
    return BaseUtils::algo::max_value(
        vertices.begin(), vertices.end(), [&](const R3& vertex) -> double {
            return (rotation ? rotation->transformed(vertex) : vertex).z();
        });
}
